<?php

namespace app\models;

use Yii;
use yii\web\IdentityInterface;
use explosivebit\arangodb\ActiveRecord;

class Account extends ActiveRecord implements IdentityInterface
{
    public $_mail;
    public $_pswd;
    public $_name;
    public $_simc;
    public $_sity;
    public $_comp;
    public $_taxn;
    public $_auth;

    /**
     * @return string the name of the index associated with this ActiveRecord class.
     */
    public static function collectionName()
    {
        return 'account';
    }

    /**
     * @return array list of attribute names.
     */
    public function attributes()
    {
        return ['mail', 'pswd', 'auth', 'name' => 'sex', 'simc', 'sity', 'comp', 'taxn', 'auth'];
    }

    /**
     * @return int|string current user ID
     */
    public function getId()
    {
        return $this->mail;
    }

    /**
     * @return string current user auth key
     */
    public function getAuthKey()
    {
        return $this->auth;
    }

    /**
     * Finds an identity by the given ID.
     *
     * @param string|int $id the ID to be looked for
     * @return IdentityInterface|null the identity object that matches the given ID.
     */
    public static function findIdentity($mail)
    {
        return static::findByMail($mail);
    }

    /**
     * Finds an identity by the given token.
     *
     * @param string $token the token to be looked for
     * @return IdentityInterface|null the identity object that matches the given token.
     */
    public static function findIdentityByAccessToken($pass, $type = null)
    {
        return static::findOne(['pass' => $pass]);
    }

    /**
     * Finds user by mail
     *
     * @param string $mail
     * @return static|null
     */
    public static function findByMail($mail)
    {
        return static::findOne(['mail' => $mail]);
    }

    /**
     * Validates mail
     *
     * @param string $pswd password to validate
     * @return bool if password provided is valid for current user
     */
    public function validateMail($mail)
    {
        if (static::findByMail($mail)) {
            // Почта найдена в базе данных

            return false;
        }

        return true;
    }

    /**
     * Validates password
     *
     * @param string $pswd password to validate
     * @return bool if password provided is valid for current user
     */
    public function validatePassword($pswd)
    {
        return Yii::$app->security->validatePassword($pswd, $this->pswd);
    }

    /**
     * @param string $auth
     * @return bool if auth key is valid for current user
     */
    public function validateAuthKey($auth)
    {
        return $this->getAuthKey() === $auth;
    }

    public function beforeSave($insert)
    {
        if (parent::beforeSave($insert)) {
            if ($this->isNewRecord) {
                $this->auth = Yii::$app->security->generateRandomString();
            }
            return true;
        }
        return false;
    }
}
