<?php

namespace app\models;

use Yii;
use yii\base\Model;
use app\models\Account;

/**
 * AccountForm is the model behind the login form.
 *
 * @property-read Account|null $account This property is read-only.
 *
 */
class AccountForm extends Model
{
    public $mail;
    public $pswd;
    public $auto = true;

    /**
     * Тип обработки
     *
     * Регистрация: 0
     * Аутентификация: 1
     *
     * @var integer
     */
    public $type = 1;

    private $account = false;


    /**
     * @return array the validation rules.
     */
    public function rules()
    {
        return [
            // Обязательные поля
            [['mail', 'pswd'], 'required', 'message' => 'Заполните поле'],
            // Функция "Запомнить меня"
            ['auto', 'boolean'],
            // Проверка почты
            ['mail', 'validateMail', 'message'=>'Неправильная почта'],
            // Проверка пароля
            ['pswd', 'validatePassword', 'message'=>'Неправильный пароль'],
        ];
    }

    public function attributeLabels()
    {
        return [
            'mail' => 'Почта',
            'pswd' => 'Пароль',
            'auto' => 'Запомнить'
        ];
    }

    /**
     * @param string $attribute the attribute currently being validated
     * @param array $params the additional name-value pairs given in the rule
     */
    public function validateMail($attribute, $params)
    {
        if (!$this->hasErrors() && $this->type === 0) {
            // Проблем нет, обрабатывается событие регистрации

            $account = $this->getAccount();

            if (!$account || !$account->validateMail($this->mail)) {
                // Проверка не пройдена
                $this->addError($attribute, 'Почта уже привязана к другому аккаунту');
            }
        }
    }

    /**
     * Validates the password.
     * This method serves as the inline validation for password.
     *
     * @param string $attribute the attribute currently being validated
     * @param array $params the additional name-value pairs given in the rule
     */
    public function validatePassword($attribute, $params)
    {
        if (!$this->hasErrors() && $this->type === 1) {
            // Проблем нет, обрабатывается событие аутентификации

            $account = $this->getAccount();

            if (!$account || !$account->validatePassword($this->pswd)) {
                // Проверка не пройдена

                $this->addError($attribute, 'Проверьте входные данные');
            }
        }
    }

    /**
     * Logs in a account using the provided accountname and password.
     * @return bool whether the account is logged in successfully
     */
    public function authentication()
    {
        if (isset($this->mail, $this->pswd) && $this->validate()) {
            // Проверка пройдена

            // Аутентификация
            return Yii::$app->user->login($this->getAccount(), $this->auto ? 3600*24*30 : 0);
        }

        return false;
    }

    /**
     * @return bool
     */
    public function registration()
    {
        // Инициализация нового аккаунта
        $this->account = new Account();

        if (isset($this->mail, $this->pswd) && $this->validate()) {
            // Проверка пройдена

            // Запись параметров
            $this->account->mail = $this->mail;
            $this->account->pswd = Yii::$app->security->generatePasswordHash($this->pswd);

            // Регистрация
            return $this->account->save();
        }

        return false;
    }

    /**
     * Finds account by [[accountname]]
     *
     * @return Account|null
     */
    public function getAccount()
    {
        if ($this->account === false) {
            $this->account = Account::findByMail($this->mail);
        }

        return $this->account;
    }
}
